@@ -1552,7 +1552,7 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	continue;
       }
     } else {
-      if (c < 80)
+      if (c < 0x80)
 	{
 	  mode = modes[c & 0x7f].modes;
 	  process = modes[c & 0x7f].process;
